@@ -25,6 +25,14 @@ from .parser_tag import parse_commit_message as tag_parser  # noqa isort:skip
 logger = logging.getLogger(__name__)
 
 
+prerelease_pattern = f"-{config.get('prerelease_tag')}\.\d+"
+version_pattern = f"(\d+\.\d+\.\d+({prerelease_pattern})?)"
+release_version_pattern = f"(\d+\.\d+\.\d+(?!.*{prerelease_pattern}))"
+
+release_version_regex = rf"{release_version_pattern}"
+version_regex = rf"{version_pattern}"
+
+
 class VersionDeclaration(ABC):
     def __init__(self, path: Union[str, Path]):
         self.path = Path(path)
@@ -45,9 +53,7 @@ class VersionDeclaration(ABC):
         variable name.
         """
         path, variable = config_str.split(":", 1)
-        pattern = (
-            rf'{variable} *[:=] *["\']{PatternVersionDeclaration.version_regex}["\']'
-        )
+        pattern = rf'{variable} *[:=] *["\']{version_regex}["\']'
         return PatternVersionDeclaration(path, pattern)
 
     @staticmethod
@@ -57,7 +63,7 @@ class VersionDeclaration(ABC):
         regular expression matching the version number.
         """
         path, pattern = config_str.split(":", 1)
-        pattern = pattern.format(version=PatternVersionDeclaration.version_regex)
+        pattern = pattern.format(version=version_regex)
         return PatternVersionDeclaration(path, pattern)
 
     @abstractmethod
@@ -116,8 +122,6 @@ class PatternVersionDeclaration(VersionDeclaration):
     function to create the version patterns specified in the config files.
     """
 
-    version_regex = r"(\d+\.\d+(?:\.\d+)?)"
-
     # The pattern should be a regular expression with a single group,
     # containing the version to replace.
     def __init__(self, path: str, pattern: str):
@@ -183,7 +187,22 @@ def get_current_version_by_tag() -> str:
 
     :return: A string with the version number or 0.0.0 on failure.
     """
-    version = get_last_version()
+    version = get_last_version(pattern=version_pattern)
+    if version:
+        return version
+
+    logger.debug("no version found, returning default of v0.0.0")
+    return "0.0.0"
+
+
+@LoggedFunction(logger)
+def get_current_release_version_by_tag() -> str:
+    """
+    Find the current version of the package in the current working directory using git tags.
+
+    :return: A string with the version number or 0.0.0 on failure.
+    """
+    version = get_last_version(pattern=release_version_pattern)
     if version:
         return version
 
@@ -219,28 +238,91 @@ def get_current_version_by_config_file() -> str:
 def get_current_version() -> str:
     """
     Get current version from tag or version variable, depending on configuration.
+    This can be either a release or prerelease version
+
+    :return: A string with the current version number
+    """
+    if config.get("version_source") in ["tag", "tag_only"]:
+        return get_current_version_by_tag()
+    else:
+        return get_current_version_by_config_file()
+
+
+def get_current_release_version() -> str:
+    """
+    Get current release version from tag or commit message (no going back in config file),
+    depending on configuration.
+    This will return the current release version (NOT prerelease), instead of just the current version
 
     :return: A string with the current version number
     """
     if config.get("version_source") == "tag":
-        return get_current_version_by_tag()
-    return get_current_version_by_config_file()
+        return get_current_release_version_by_tag()
+    else:
+        return get_current_release_version_by_commits()
 
 
 @LoggedFunction(logger)
-def get_new_version(current_version: str, level_bump: str) -> str:
+def get_new_version(
+    current_version: str,
+    current_release_version: str,
+    level_bump: str,
+    prerelease: bool = False,
+) -> str:
     """
     Calculate the next version based on the given bump level with semver.
 
     :param current_version: The version the package has now.
     :param level_bump: The level of the version number that should be bumped.
         Should be `'major'`, `'minor'` or `'patch'`.
+    :param prerelease: Should the version bump be marked as a prerelease
     :return: A string with the next version number.
     """
-    if not level_bump:
-        logger.debug("No bump requested, returning input version")
-        return current_version
-    return str(semver.VersionInfo.parse(current_version).next_version(part=level_bump))
+    # pre or release version
+    current_version_info = semver.VersionInfo.parse(current_version)
+    # release version
+    current_release_version_info = semver.VersionInfo.parse(current_release_version)
+
+    # sanity check
+    # if the current version is no prerelease, than
+    # current_version and current_release_version must be the same
+    if (
+        not current_version_info.prerelease
+        and current_version_info.compare(current_release_version_info) != 0
+    ):
+        raise ValueError(
+            f"Error: Current version {current_version} and current release version {current_release_version} do not match!"
+        )
+
+    if level_bump:
+        next_version_info = current_release_version_info.next_version(level_bump)
+    elif prerelease:
+        # we do at least a patch for prereleases
+        next_version_info = current_release_version_info.next_version("patch")
+    else:
+        next_version_info = current_release_version_info
+
+    if prerelease:
+        next_raw_version = next_version_info.to_tuple()[:3]
+        current_raw_version = current_version_info.to_tuple()[:3]
+
+        if current_version_info.prerelease and next_raw_version == current_raw_version:
+            # next version (based on commits) matches current prerelease version
+            # bump prerelase
+            next_prerelease_version_info = current_version_info.bump_prerelease(
+                config.get("prerelease_tag")
+            )
+        else:
+            # next version (based on commits) higher than current prerelease version
+            # new prerelease based on next version
+            next_prerelease_version_info = next_version_info.bump_prerelease(
+                config.get("prerelease_tag")
+            )
+
+        return str(next_prerelease_version_info)
+    else:
+        # normal version bump
+        return str(next_version_info)
 
 
 @LoggedFunction(logger)
@@ -260,12 +342,59 @@ def get_previous_version(version: str) -> Optional[str]:
             continue
 
         if found_version:
-            matches = re.match(r"v?(\d+.\d+.\d+)", commit_message)
-            if matches:
+            match = re.search(rf"{version_pattern}", commit_message)
+            if match:
                 logger.debug(f"Version matches regex {commit_message}")
-                return matches.group(1).strip()
+                return match.group(1).strip()
 
-    return get_last_version([version, get_formatted_tag(version)])
+    return get_last_version(
+        pattern=version_pattern, skip_tags=[version, get_formatted_tag(version)]
+    )
+
+
+@LoggedFunction(logger)
+def get_previous_release_version(version: str) -> Optional[str]:
+    """
+    Return the version prior to the given version.
+
+    :param version: A string with the version number.
+    :return: A string with the previous version number.
+    """
+    found_version = False
+    for commit_hash, commit_message in get_commit_log():
+        logger.debug(f"Checking commit {commit_hash}")
+        if version in commit_message:
+            found_version = True
+            logger.debug(f'Found version in commit "{commit_message}"')
+            continue
+
+        if found_version:
+            match = re.search(rf"{release_version_pattern}", commit_message)
+            if match:
+                logger.debug(f"Version matches regex {commit_message}")
+                return match.group(1).strip()
+
+    return get_last_version(
+        pattern=release_version_pattern, skip_tags=[version, get_formatted_tag(version)]
+    )
+
+
+@LoggedFunction(logger)
+def get_current_release_version_by_commits() -> str:
+    """
+    Return the current release version (NOT prerelease) version.
+
+    :return: A string with the current version number.
+    """
+    for commit_hash, commit_message in get_commit_log():
+        logger.debug(f"Checking commit {commit_hash}")
+        match = re.search(rf"{release_version_pattern}", commit_message)
+        if match:
+            logger.debug(f"Version matches regex {commit_message}")
+            return match.group(1).strip()
+
+    # nothing found, return the initial version
+    return "0.0.0"
 
 
 @LoggedFunction(logger)
