@@ -63,7 +63,7 @@ def get_commit_log(from_rev=None):
 
 @check_repo
 @LoggedFunction(logger)
-def get_last_version(skip_tags=None, omit_pattern=None) -> Optional[str]:
+def get_last_version(pattern, skip_tags=None) -> Optional[str]:
     """
     Find the latest version using repo tags.
 
@@ -77,32 +77,16 @@ def get_last_version(skip_tags=None, omit_pattern=None) -> Optional[str]:
         return tag.commit.committed_date
 
     for i in sorted(repo.tags, reverse=True, key=version_finder):
-        match = re.search(r"\d+\.\d+\.\d+", i.name)
+        if i.name in skip_tags:
+            continue
+
+        match = re.search(rf"{pattern}", i.name)
         if match:
-            # check if the omit pattern is present in the tag (e.g. -beta for pre-release tags)
-            if omit_pattern and omit_pattern in i.name:
-                continue
-            if i.name not in skip_tags:
-                return match.group(0)  # Return only numeric vesion like 1.2.3
+            return match.group(0).strip()
 
     return None
 
 
-@check_repo
-@LoggedFunction(logger)
-def get_version_from_tag(tag_name: str) -> Optional[str]:
-    """
-    Get the git commit hash corresponding to a tag name.
-
-    :param tag_name: Name of the git tag (i.e. 'v1.0.0')
-    :return: sha1 hash of the commit
-    """
-    for i in repo.tags:
-        if i.name == tag_name:
-            return i.commit.hexsha
-    return None
-
-
 @check_repo
 @LoggedFunction(logger)
 def get_repository_owner_and_name() -> Tuple[str, str]:
@@ -277,14 +261,17 @@ def push_new_version(
     """
     server = "origin"
     if auth_token:
-        token = auth_token
-        if config.get("hvcs") == "gitlab":
-            token = "gitlab-ci-token:" + token
-        actor = os.environ.get("GITHUB_ACTOR")
-        if actor:
-            server = f"https://{actor}:{token}@{domain}/{owner}/{name}.git"
+        if not config.get("ignore_token_for_push"):
+            token = auth_token
+            if config.get("hvcs") == "gitlab":
+                token = "gitlab-ci-token:" + token
+            actor = os.environ.get("GITHUB_ACTOR")
+            if actor:
+                server = f"https://{actor}:{token}@{domain}/{owner}/{name}.git"
+            else:
+                server = f"https://{token}@{domain}/{owner}/{name}.git"
         else:
-            server = f"https://{token}@{domain}/{owner}/{name}.git"
+            logger.debug("Ignoring token for pushing to the repository.")
 
     try:
         repo.git.push(server, branch)
